package ip;

import java.math.BigInteger;
import java.util.StringTokenizer;

/** This class is used to manage a set of IP Address, Network Mask, and Default Gateway
    IP Address.  It includes methods for converting between an int and Strings that
    represent the dot-notation of an IP Address. */
public class IPAddress {
   public static final String DEFAULT_IP="0.0.0.0"; 

/** The IP Address */
   int ipAddress=0;

/** The Network Mask */
   int netMask=0;

/** The Defaut Gateway IP */
   int defaultGateway=0;

/** Creates an IPAddress with all 3 values set to 0. */
   public IPAddress() {
      ipAddress=0;
      netMask=0;
      defaultGateway=0;
   }

/** Creates an IPAddress object from the three Strings. */
   public IPAddress(String address,String mask,String gateway) {
      if(address!=null) ipAddress=parseIPAddr(address);
      if(mask!=null) netMask=parseIPAddr(mask);
      if(gateway!=null) defaultGateway=parseIPAddr(gateway);
   }

/** Creates an IPAddress Object from 3 integers. */
   public IPAddress(int address,int mask,int gateway) {
      ipAddress=address;
      netMask=mask;
      defaultGateway=gateway;
   }

/** Creates an IPAddress Object from 3 byte arrays. */
   public IPAddress(byte[] address,byte[] mask,byte[] gateway) {
      ipAddress=toInt(address);
      netMask=toInt(mask);
      defaultGateway=toInt(gateway);
   }

/** Returns the number of IP Addresses that the Network Mask can resolve. */
   public int getAddressCount() {
      return -netMask;
   }

/** Converts a byte array to an int. */
   public static int toInt(byte[] b) {
      return new BigInteger(b).intValue();
   }

/** Returns the IP Address. */
   public int getIPAddress() {return ipAddress;}

/** Returns the Network Mask. */
   public int getNetMask() {return netMask;}

/** Returns the Defaut Gateway IP Address. */
   public int getDefaultGateway() {return defaultGateway;}

/** Returns the IP Address as a String. */
   public String getIPAddressString() {return intToIPString(ipAddress);}

/** Returns the Network Mask as a String. */
   public String getNetMaskString() {return intToIPString(netMask);}

/** Returns the Defaut Gateway IP Address as a String. */
   public String getDefaultGatewayString() {return intToIPString(defaultGateway);}

/** Sets the IP Address. */
   public void setIPAddress(int addr) {ipAddress=addr;}

/** Sets the Network Mask. */
   public void setNetMask(int addr) {netMask=addr;}

/** Sets the Default Gateway IP Address. */
   public void setDefaultGateway(int addr) {defaultGateway=addr;}

/** Sets the IP Address. */
   public void setIPAddress(String addr) {ipAddress=parseIPAddr(addr);}

/** Sets the Network Mask. */
   public void setNetMask(String addr) {netMask=parseIPAddr(addr);}

/** Sets the Default Gateway IP Address. */
   public void setDefaultGateway(String addr) {defaultGateway=parseIPAddr(addr);}

/** Returns true if ip has the same IP Address and Network Mask as this IPAddress
    Object.*/
   public boolean equals(IPAddress ip) {
      return equals(ip,false);
   }

/** If gateway is true, returns true if the IP Address, mask, and gateway of ip
    is the same as this IPAddress.  If gateway is false, ignores the gateway
    comparison.*/
   public boolean equals(IPAddress ip,boolean gateway) {
      boolean result=false;
      if(ip.getIPAddress()==ipAddress && ip.getNetMask()==netMask) {
         if(gateway) result=ip.getDefaultGateway()==defaultGateway;
         else result=true;
      }
      return result;
   }

/** Makes a Default Gateway address that is the next-highest address from the base
    IP Address. */
   public static String makeDefaultGateway(String ip) throws NumberFormatException {
      int addr=parseIPAddr(ip);
      return intToIPString(++addr);
   }

/** Converts a String representation of an IP Address into an int. */
   public static int parseIPAddr(String address) throws NumberFormatException {
      StringTokenizer st=new StringTokenizer(address.trim(),".");
      if(st.countTokens()==4) {
         byte[] b=new byte[4];
         for(int i=0;i<b.length;i++) {
            int temp=Integer.parseInt(st.nextToken());
            if(temp>255) throw (new NumberFormatException()); 
            b[i]=(byte) (temp & 0xff);
         }
         return new BigInteger(b).intValue();
      } else throw (new NumberFormatException());
   }

/** Converts an int to a byte array. */
   public static byte[] toByteArray(int address) {
      byte[] b=new byte[4];
      for(int i=0;i<b.length;i++) {
         b[3-i]=(byte) (address & 0xff);
         address>>=8;
      }
      return b;
   }

/** Converts an int to a String representing an IP Address. */
   public static String intToIPString(int address) {
      byte[] b=toByteArray(address);
      String result=new String("");
      for(int i=0;i<b.length;i++) {
         if(i!=0) result=result+".";
         result=result+Integer.toString(b[i] & 0xff);
      }
      return result;
   }

/* Returns a String with all 3 IP variables. */
   public String toString() {
      String ip=intToIPString(ipAddress);
      ip=ip+","+intToIPString(netMask);
      ip=ip+","+intToIPString(defaultGateway);
      return ip;
   }

   public static void main(String[] args) {
      IPAddress ip=new IPAddress("192.168.0.123","255.255.255.192","192.155.3.1");
      System.out.println(ip.getIPAddress()+"  "+ip.getNetMask());
      System.out.println(ip.toString());
      System.out.println(ip.getAddressCount());
   }
} 
