package ip;

import java.util.Vector; 
import java.math.BigInteger;

public class Subnet extends IPAddress {
   private int addrCount=0;
   private Subnet child0=null;
   private Subnet child1=null;
   private boolean available=true;

   public Subnet(int addr,int mask, int gateway) {
      super(addr,mask,gateway);
      addrCount=-getNetMask();
      ipAddress=addr & mask;
   }

   public Subnet(String address,String mask,String gateway) {
      super(address,mask,gateway);
      addrCount=-getNetMask();
      ipAddress=ipAddress & netMask;
   }

   public void setCount(int count) {addrCount=count;}

   public int getCount() {return addrCount;}

   public Subnet getChild0() {return child0;}

   public Subnet getChild1() {return child1;}

   public boolean isAvailable() {return available;}

   public boolean hasSubnet() {return child0!=null;}

   public void setAvailable(boolean b) {available=b;}

   void split() {
      int count=addrCount>>>1;
      int mask=netMask>>1;
      child0=new Subnet(ipAddress,mask,defaultGateway);
      child1=new Subnet(ipAddress | count,mask,defaultGateway);
   }

   Subnet allocate(Subnet s,int count) {
      Subnet result=null;
      if(s.isAvailable()) {
         int testcount=s.getCount();
         if(testcount==count && !s.hasSubnet()) {
            s.setAvailable(false);
            result=s;
         } else if(testcount>count) result=s.allocate(count);
      }
      return result;
   }

   public Subnet allocate(int count) {
      Subnet result=null;

/* If count is too high or this isn't available return null. */
      if(count>addrCount || !isAvailable()) return result;


/* If subnet won't work, return this. */
      if(count>addrCount>>>1) {
         if(child0!=null) return result;
         setAvailable(false);
         return this;
      } 

/* Subnet if available. */ 
      if(count>3) {
         BigInteger bi=new BigInteger(toByteArray(--count));
         int newcount=(1<<bi.bitLength());
         //int mask=~bits;
         if(child0==null) split();
         result=allocate(child1,newcount);
         if(result==null) result=allocate(child0,newcount);
         if(!child0.isAvailable() && !child1.isAvailable()) setAvailable(false);
      }
      return result;
   } 

/** Returns true if the IP Address is part of this subnet. */
   public boolean contains(IPAddress addr) {
      return (ipAddress==(netMask & addr.getIPAddress()));
   }

/** Adds a subnet to a child of this subnet, if the subnet is contained in
    this subnet.  Overwrites any existing subnet to add the new one.  This
    includes parent subnets that may be already allocated.*/
   public boolean addSubnet(Subnet sn) {
      if(!contains(sn)) return false;
      boolean result=false;
      if(equals(sn,false)) {
         child0=null;
         child1=null;
         setDefaultGateway(sn.getDefaultGateway());
         setAvailable(false);
         return true;
      } else {
         if(!hasSubnet()) split();
         result=child1.addSubnet(sn);
         if(!result) result=child0.addSubnet(sn);
         setAvailable(child0.isAvailable() || child1.isAvailable());
      }
      return result;
   }

/** Removes a child subnet from this subnet.*/
   public boolean deleteSubnet(Subnet sn) {
      if(!contains(sn)) return false;
      if(equals(sn,false)) return false;
      if(!hasSubnet()) return false;
      boolean result=false;
      Subnet c0=getChild0();
      Subnet c1=getChild1();
      if(c1.equals(sn,false)) {
         child1.setAvailable(true);
         setAvailable(true);
         return true;
      } else if(c0.equals(sn,false)) {
         child0.setAvailable(true);
         setAvailable(true);
         return true;
      }
      if(c1.deleteSubnet(sn)) return true;
      if(c0.deleteSubnet(sn)) return true;
      return false;
   }

   public static void main(String[] args) {
      Subnet sn=new Subnet(0xc0010200,0xffffff00,0);
      Subnet sn2=new Subnet(0xc00102fc,0xfffffffc,0);
      Subnet sn3=new Subnet(0xc00102f8,0xfffffffc,0);
      IPAddress sn4=new IPAddress(0xc00102fd,0xfffffffc,0);
      IPAddress sn5=new IPAddress(0xc00102f9,0xfffffffc,0);

      System.out.println(sn2.getIPAddressString()+"  "+sn.contains(sn2));
      System.out.println(sn4.getIPAddressString()+"  "+sn2.contains(sn4));
      System.out.println(sn5.getIPAddressString()+"  "+sn3.contains(sn5));
      System.out.println(sn5.getIPAddressString()+"  "+sn2.contains(sn5));
      System.out.println(sn4.getIPAddressString()+"  "+sn3.contains(sn4));
   }
} 
